@@ -445,12 +445,40 @@ mod typed_function;
 mod value;
 pub mod vm;
 
+#[cfg(any(feature = "wasm-types-polyfill", feature = "jsc"))]
+mod module_info_polyfill;
+
 #[cfg(feature = "sys")]
-mod sys;
+/// sys
+pub mod sys;
 
 #[cfg(feature = "sys")]
 pub use sys::*;
 
+#[cfg(feature = "sys")]
+#[deprecated(note = "wasmer::Artifact is deprecated, use wasmer::sys::Artifact instead")]
+/// A compiled wasm module, ready to be instantiated.
+pub type Artifact = sys::Artifact;
+#[cfg(feature = "sys")]
+#[deprecated(note = "wasmer::EngineBuilder is deprecated, use wasmer::sys::EngineBuilder instead")]
+/// The Builder contents of `Engine`
+pub type EngineBuilder = sys::EngineBuilder;
+#[cfg(feature = "sys")]
+#[deprecated(note = "wasmer::Features is deprecated, use wasmer::sys::Features instead")]
+/// Controls which experimental features will be enabled.
+pub type Features = sys::Features;
+#[cfg(feature = "sys")]
+#[deprecated(note = "wasmer::BaseTunables is deprecated, use wasmer::sys::BaseTunables instead")]
+/// Tunable parameters for WebAssembly compilation.
+/// This is the reference implementation of the `Tunables` trait,
+/// used by default.
+pub type BaseTunables = sys::BaseTunables;
+#[cfg(feature = "sys")]
+#[deprecated(note = "wasmer::VMConfig is deprecated, use wasmer::sys::VMConfig instead")]
+/// Configuration for the the runtime VM
+/// Currently only the stack size is configurable
+pub type VMConfig = sys::VMConfig;
+
 #[cfg(feature = "js")]
 mod js;
 
@@ -463,10 +491,12 @@ mod jsc;
 #[cfg(feature = "jsc")]
 pub use jsc::*;
 
-pub use crate::externals::{Extern, Function, Global, HostFunction, Memory, MemoryView, Table};
+pub use crate::externals::{
+    Extern, Function, Global, HostFunction, Memory, MemoryLocation, MemoryView, SharedMemory, Table,
+};
 pub use access::WasmSliceAccess;
 pub use engine::{AsEngineRef, Engine, EngineRef};
-pub use errors::{InstantiationError, LinkError, RuntimeError};
+pub use errors::{AtomicsError, InstantiationError, LinkError, RuntimeError};
 pub use exports::{ExportError, Exportable, Exports, ExportsIterator};
 pub use extern_ref::ExternRef;
 pub use function_env::{FunctionEnv, FunctionEnvMut};
