@@ -183,8 +183,8 @@ unsafe fn fmt_time(ctx: FunctionEnvMut<EmEnv>, time: u32) -> *const c_char {
     let memory = ctx.data().memory(0);
     let date = &*(emscripten_memory_pointer!(memory.view(&ctx), time) as *mut guest_tm);
 
-    let days = vec!["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"];
-    let months = vec![
+    let days = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"];
+    let months = [
         "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
     ];
     let year = 1900 + date.tm_year;
@@ -426,7 +426,8 @@ pub fn _strftime(
     ) else {
         return 0;
     };
-    let Ok(rust_time) = time::Time::from_hms(tm.tm_hour as u8, tm.tm_min as u8, tm.tm_sec as u8) else {
+    let Ok(rust_time) = time::Time::from_hms(tm.tm_hour as u8, tm.tm_min as u8, tm.tm_sec as u8)
+    else {
         return 0;
     };
     let rust_datetime = time::PrimitiveDateTime::new(rust_date, rust_time);
