@@ -3,6 +3,7 @@ use std::mem;
 use wasmer::FromToNativeWasmType;
 // use std::ffi::CStr;
 use std::os::raw::c_char;
+use wasmer::FunctionEnvMut;
 
 #[repr(transparent)]
 #[derive(Copy, Clone)]
@@ -11,18 +12,18 @@ pub struct VarArgs {
 }
 
 impl VarArgs {
-    pub fn get<T: Sized>(&mut self, ctx: &EmEnv) -> T {
-        let ptr = emscripten_memory_pointer!(ctx.memory(0), self.pointer);
+    pub fn get<T: Sized>(&mut self, ctx: &FunctionEnvMut<EmEnv>) -> T {
+        let memory = ctx.data().memory(0);
+        let ptr = emscripten_memory_pointer!(memory.view(&ctx), self.pointer);
         self.pointer += mem::size_of::<T>() as u32;
         unsafe { (ptr as *const T).read() }
     }
 
     // pub fn getStr<'a>(&mut self, ctx: &mut Ctx) -> &'a CStr {
-    pub fn get_str(&mut self, ctx: &EmEnv) -> *const c_char {
+    pub fn get_str(&mut self, ctx: &FunctionEnvMut<EmEnv>) -> *const c_char {
         let ptr_addr: u32 = self.get(ctx);
-        let ptr = emscripten_memory_pointer!(ctx.memory(0), ptr_addr) as *const c_char;
-        ptr
-        // unsafe { CStr::from_ptr(ptr) }
+        let memory = ctx.data().memory(0);
+        emscripten_memory_pointer!(memory.view(&ctx), ptr_addr) as *const c_char
     }
 }
 
