@@ -105,6 +105,110 @@ pub struct Relocation {
     pub addend: Addend,
 }
 
+/// Any struct that acts like a `Relocation`.
+#[allow(missing_docs)]
+pub trait RelocationLike {
+    fn kind(&self) -> RelocationKind;
+    fn reloc_target(&self) -> RelocationTarget;
+    fn offset(&self) -> CodeOffset;
+    fn addend(&self) -> Addend;
+
+    /// Given a function start address, provide the relocation relative
+    /// to that address.
+    ///
+    /// The function returns the relocation address and the delta.
+    fn for_address(&self, start: usize, target_func_address: u64) -> (usize, u64) {
+        match self.kind() {
+            RelocationKind::Abs8
+            | RelocationKind::Arm64Movw0
+            | RelocationKind::Arm64Movw1
+            | RelocationKind::Arm64Movw2
+            | RelocationKind::Arm64Movw3
+            | RelocationKind::RiscvPCRelLo12I => {
+                let reloc_address = start + self.offset() as usize;
+                let reloc_addend = self.addend() as isize;
+                let reloc_abs = target_func_address
+                    .checked_add(reloc_addend as u64)
+                    .unwrap();
+                (reloc_address, reloc_abs)
+            }
+            RelocationKind::X86PCRel4 => {
+                let reloc_address = start + self.offset() as usize;
+                let reloc_addend = self.addend() as isize;
+                let reloc_delta_u32 = (target_func_address as u32)
+                    .wrapping_sub(reloc_address as u32)
+                    .checked_add(reloc_addend as u32)
+                    .unwrap();
+                (reloc_address, reloc_delta_u32 as u64)
+            }
+            RelocationKind::X86PCRel8 => {
+                let reloc_address = start + self.offset() as usize;
+                let reloc_addend = self.addend() as isize;
+                let reloc_delta = target_func_address
+                    .wrapping_sub(reloc_address as u64)
+                    .checked_add(reloc_addend as u64)
+                    .unwrap();
+                (reloc_address, reloc_delta)
+            }
+            RelocationKind::X86CallPCRel4 | RelocationKind::X86CallPLTRel4 => {
+                let reloc_address = start + self.offset() as usize;
+                let reloc_addend = self.addend() as isize;
+                let reloc_delta_u32 = (target_func_address as u32)
+                    .wrapping_sub(reloc_address as u32)
+                    .wrapping_add(reloc_addend as u32);
+                (reloc_address, reloc_delta_u32 as u64)
+            }
+            RelocationKind::Arm64Call
+            | RelocationKind::RiscvCall
+            | RelocationKind::RiscvPCRelHi20 => {
+                let reloc_address = start + self.offset() as usize;
+                let reloc_addend = self.addend() as isize;
+                let reloc_delta_u32 = target_func_address
+                    .wrapping_sub(reloc_address as u64)
+                    .wrapping_add(reloc_addend as u64);
+                (reloc_address, reloc_delta_u32)
+            }
+            _ => panic!("Relocation kind unsupported"),
+        }
+    }
+}
+
+impl RelocationLike for Relocation {
+    fn kind(&self) -> RelocationKind {
+        self.kind
+    }
+
+    fn reloc_target(&self) -> RelocationTarget {
+        self.reloc_target
+    }
+
+    fn offset(&self) -> CodeOffset {
+        self.offset
+    }
+
+    fn addend(&self) -> Addend {
+        self.addend
+    }
+}
+
+impl RelocationLike for ArchivedRelocation {
+    fn kind(&self) -> RelocationKind {
+        self.kind
+    }
+
+    fn reloc_target(&self) -> RelocationTarget {
+        self.reloc_target
+    }
+
+    fn offset(&self) -> CodeOffset {
+        self.offset
+    }
+
+    fn addend(&self) -> Addend {
+        self.addend
+    }
+}
+
 /// Destination function. Can be either user function or some special one, like `memory.grow`.
 #[cfg_attr(feature = "enable-serde", derive(Serialize, Deserialize))]
 #[derive(
@@ -121,66 +225,5 @@ pub enum RelocationTarget {
     CustomSection(SectionIndex),
 }
 
-impl Relocation {
-    /// Given a function start address, provide the relocation relative
-    /// to that address.
-    ///
-    /// The function returns the relocation address and the delta.
-    pub fn for_address(&self, start: usize, target_func_address: u64) -> (usize, u64) {
-        match self.kind {
-            RelocationKind::Abs8
-            | RelocationKind::Arm64Movw0
-            | RelocationKind::Arm64Movw1
-            | RelocationKind::Arm64Movw2
-            | RelocationKind::Arm64Movw3
-            | RelocationKind::RiscvPCRelLo12I => {
-                let reloc_address = start + self.offset as usize;
-                let reloc_addend = self.addend as isize;
-                let reloc_abs = target_func_address
-                    .checked_add(reloc_addend as u64)
-                    .unwrap();
-                (reloc_address, reloc_abs)
-            }
-            RelocationKind::X86PCRel4 => {
-                let reloc_address = start + self.offset as usize;
-                let reloc_addend = self.addend as isize;
-                let reloc_delta_u32 = (target_func_address as u32)
-                    .wrapping_sub(reloc_address as u32)
-                    .checked_add(reloc_addend as u32)
-                    .unwrap();
-                (reloc_address, reloc_delta_u32 as u64)
-            }
-            RelocationKind::X86PCRel8 => {
-                let reloc_address = start + self.offset as usize;
-                let reloc_addend = self.addend as isize;
-                let reloc_delta = target_func_address
-                    .wrapping_sub(reloc_address as u64)
-                    .checked_add(reloc_addend as u64)
-                    .unwrap();
-                (reloc_address, reloc_delta)
-            }
-            RelocationKind::X86CallPCRel4 | RelocationKind::X86CallPLTRel4 => {
-                let reloc_address = start + self.offset as usize;
-                let reloc_addend = self.addend as isize;
-                let reloc_delta_u32 = (target_func_address as u32)
-                    .wrapping_sub(reloc_address as u32)
-                    .wrapping_add(reloc_addend as u32);
-                (reloc_address, reloc_delta_u32 as u64)
-            }
-            RelocationKind::Arm64Call
-            | RelocationKind::RiscvCall
-            | RelocationKind::RiscvPCRelHi20 => {
-                let reloc_address = start + self.offset as usize;
-                let reloc_addend = self.addend as isize;
-                let reloc_delta_u32 = target_func_address
-                    .wrapping_sub(reloc_address as u64)
-                    .wrapping_add(reloc_addend as u64);
-                (reloc_address, reloc_delta_u32)
-            }
-            _ => panic!("Relocation kind unsupported"),
-        }
-    }
-}
-
 /// Relocations to apply to function bodies.
 pub type Relocations = PrimaryMap<LocalFunctionIndex, Vec<Relocation>>;
