@@ -76,7 +76,10 @@ pub use crate::compilation::target::{
     Aarch64Architecture, Architecture, BinaryFormat, CallingConvention, CpuFeature, Endianness,
     Environment, OperatingSystem, PointerWidth, Target, Triple, Vendor,
 };
-pub use crate::serialize::{MetadataHeader, SerializableCompilation, SerializableModule};
+pub use crate::serialize::{
+    ArchivedSerializableCompilation, ArchivedSerializableModule, MetadataHeader,
+    SerializableCompilation, SerializableModule,
+};
 pub use error::{
     CompileError, DeserializeError, ImportError, MemoryError, MiddlewareError,
     ParseCpuFeatureError, PreInstantiationError, SerializeError, WasmError, WasmResult,
@@ -91,7 +94,9 @@ pub use crate::indexes::{
     SignatureIndex, TableIndex,
 };
 pub use crate::initializers::{
-    DataInitializer, DataInitializerLocation, OwnedDataInitializer, TableInitializer,
+    ArchivedDataInitializerLocation, ArchivedOwnedDataInitializer, DataInitializer,
+    DataInitializerLike, DataInitializerLocation, DataInitializerLocationLike,
+    OwnedDataInitializer, TableInitializer,
 };
 pub use crate::memory::{Memory32, Memory64, MemorySize};
 pub use crate::module::{ExportsIterator, ImportKey, ImportsIterator, ModuleInfo};
@@ -114,20 +119,24 @@ pub use crate::vmoffsets::{TargetSharedSignatureIndex, VMBuiltinFunctionIndex, V
 pub use crate::utils::is_wasm;
 
 pub use crate::compilation::relocation::{
-    Relocation, RelocationKind, RelocationTarget, Relocations,
+    ArchivedRelocation, Relocation, RelocationKind, RelocationLike, RelocationTarget, Relocations,
 };
 pub use crate::compilation::section::{
-    CustomSection, CustomSectionProtection, SectionBody, SectionIndex,
+    ArchivedCustomSection, CustomSection, CustomSectionLike, CustomSectionProtection, SectionBody,
+    SectionIndex,
 };
 
 pub use crate::compilation::address_map::{FunctionAddressMap, InstructionAddressMap};
 pub use crate::compilation::function::{
-    Compilation, CompiledFunction, CompiledFunctionFrameInfo, CustomSections, Dwarf, FunctionBody,
-    Functions,
+    ArchivedFunctionBody, Compilation, CompiledFunction, CompiledFunctionFrameInfo, CustomSections,
+    Dwarf, FunctionBody, FunctionBodyLike, Functions,
 };
 pub use crate::compilation::module::CompileModuleInfo;
 pub use crate::compilation::symbols::{Symbol, SymbolRegistry};
-pub use crate::compilation::unwind::CompiledFunctionUnwindInfo;
+pub use crate::compilation::unwind::{
+    ArchivedCompiledFunctionUnwindInfo, CompiledFunctionUnwindInfo, CompiledFunctionUnwindInfoLike,
+    CompiledFunctionUnwindInfoReference,
+};
 
 pub use crate::stack::{FrameInfo, SourceLoc, TrapInformation};
 pub use crate::store_id::StoreId;
