@@ -12,7 +12,7 @@ use std::{
 };
 use tokio::io::{AsyncRead, AsyncSeek, AsyncWrite};
 
-#[derive(Derivative, Clone)]
+#[derive(Derivative)]
 #[derivative(Debug)]
 pub struct ArcFile<T>
 where
@@ -153,6 +153,17 @@ where
     }
 }
 
+impl<T> Clone for ArcFile<T>
+where
+    T: VirtualFile + Send + Sync + 'static,
+{
+    fn clone(&self) -> Self {
+        ArcFile {
+            inner: self.inner.clone(),
+        }
+    }
+}
+
 impl<T> ClonableVirtualFile for ArcFile<T>
 where
     T: VirtualFile + Send + Sync + 'static,
