@@ -1,35 +1,37 @@
 #![cfg_attr(docsrs, feature(doc_cfg, doc_auto_cfg))]
 
-#[cfg(any(feature = "remote"))]
+#[cfg(feature = "remote")]
 pub mod client;
 #[cfg(feature = "host-net")]
 pub mod host;
 pub mod meta;
-#[cfg(any(feature = "remote"))]
+#[cfg(feature = "remote")]
 pub mod rx_tx;
-#[cfg(any(feature = "remote"))]
+#[cfg(feature = "remote")]
 pub mod server;
 #[cfg(feature = "tokio")]
 #[cfg(test)]
 mod tests;
 
-#[cfg(any(feature = "remote"))]
+#[cfg(feature = "remote")]
 pub use client::{RemoteNetworkingClient, RemoteNetworkingClientDriver};
 use pin_project_lite::pin_project;
-#[cfg(any(feature = "remote"))]
+#[cfg(feature = "rkyv")]
+use rkyv::{Archive, CheckBytes, Deserialize as RkyvDeserialize, Serialize as RkyvSerialize};
+#[cfg(feature = "remote")]
 pub use server::{RemoteNetworkingServer, RemoteNetworkingServerDriver};
 use std::fmt;
 use std::mem::MaybeUninit;
-pub use std::net::IpAddr;
-pub use std::net::Ipv4Addr;
-pub use std::net::Ipv6Addr;
+use std::net::IpAddr;
+use std::net::Ipv4Addr;
+use std::net::Ipv6Addr;
 use std::net::Shutdown;
-pub use std::net::SocketAddr;
+use std::net::SocketAddr;
 use std::pin::Pin;
 use std::sync::Arc;
 use std::task::Context;
 use std::task::Poll;
-pub use std::time::Duration;
+use std::time::Duration;
 use thiserror::Error;
 #[cfg(feature = "tokio")]
 use tokio::io::AsyncRead;
@@ -47,6 +49,8 @@ pub type Result<T> = std::result::Result<T, NetworkError>;
 
 /// Represents an IP address and its netmask
 #[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd, Serialize, Deserialize)]
+#[cfg_attr(feature = "rkyv", derive(RkyvSerialize, RkyvDeserialize, Archive))]
+#[cfg_attr(feature = "rkyv", archive_attr(derive(CheckBytes)))]
 pub struct IpCidr {
     pub ip: IpAddr,
     pub prefix: u8,
@@ -54,6 +58,8 @@ pub struct IpCidr {
 
 /// Represents a routing entry in the routing table of the interface
 #[derive(Clone, Debug, Serialize, Deserialize)]
+#[cfg_attr(feature = "rkyv", derive(RkyvSerialize, RkyvDeserialize, Archive))]
+#[cfg_attr(feature = "rkyv", archive_attr(derive(CheckBytes)))]
 pub struct IpRoute {
     pub cidr: IpCidr,
     pub via_router: IpAddr,
