@@ -1,10 +1,14 @@
 use crate::{store::MaybeInstanceOwned, vmcontext::VMGlobalDefinition};
+use derivative::Derivative;
 use std::{cell::UnsafeCell, ptr::NonNull};
 use wasmer_types::GlobalType;
 
 /// A Global instance
+#[derive(Derivative)]
+#[derivative(Debug)]
 pub struct VMGlobal {
     ty: GlobalType,
+    #[derivative(Debug = "ignore")]
     vm_global_definition: MaybeInstanceOwned<VMGlobalDefinition>,
 }
 
@@ -30,4 +34,16 @@ impl VMGlobal {
     pub fn vmglobal(&self) -> NonNull<VMGlobalDefinition> {
         self.vm_global_definition.as_ptr()
     }
+
+    /// Copies this global
+    pub fn copy_on_write(&self) -> Self {
+        unsafe {
+            Self {
+                ty: self.ty,
+                vm_global_definition: MaybeInstanceOwned::Host(Box::new(UnsafeCell::new(
+                    self.vm_global_definition.as_ptr().as_ref().clone(),
+                ))),
+            }
+        }
+    }
 }
