@@ -276,6 +276,7 @@ pub enum WasiFsRoot {
 
 impl WasiFsRoot {
     /// Merge the contents of a filesystem into this one.
+    #[tracing::instrument(level = "debug", skip_all)]
     pub(crate) async fn merge(
         &self,
         other: &Arc<dyn FileSystem + Send + Sync>,
@@ -351,7 +352,7 @@ impl FileSystem for WasiFsRoot {
 
 /// Merge the contents of one filesystem into another.
 ///
-#[tracing::instrument(level = "debug", skip_all)]
+#[tracing::instrument(level = "trace", skip_all)]
 async fn merge_filesystems(
     source: &dyn FileSystem,
     destination: &dyn FileSystem,
@@ -429,6 +430,11 @@ pub struct WasiFs {
     // but it shouldn't be necessary
     // It should not be necessary at all.
     is_wasix: AtomicBool,
+
+    // The preopens when this was initialized
+    pub(crate) init_preopens: Vec<PreopenedDir>,
+    // The virtual file system preopens when this was initialized
+    pub(crate) init_vfs_preopens: Vec<String>,
 }
 
 impl WasiFs {
@@ -454,6 +460,8 @@ impl WasiFs {
             root_fs: self.root_fs.clone(),
             root_inode: self.root_inode.clone(),
             has_unioned: Arc::new(Mutex::new(HashSet::new())),
+            init_preopens: self.init_preopens.clone(),
+            init_vfs_preopens: self.init_vfs_preopens.clone(),
         }
     }
 
@@ -496,14 +504,7 @@ impl WasiFs {
             return Ok(());
         }
 
-        match self.root_fs {
-            WasiFsRoot::Sandbox(ref sandbox_fs) => {
-                sandbox_fs.union(&binary.webc_fs);
-            }
-            WasiFsRoot::Backing(ref fs) => {
-                merge_filesystems(&binary.webc_fs, fs.deref()).await?;
-            }
-        }
+        self.root_fs.merge(&binary.webc_fs).await?;
 
         Ok(())
     }
@@ -515,175 +516,10 @@ impl WasiFs {
         vfs_preopens: &[String],
         fs_backing: WasiFsRoot,
     ) -> Result<Self, String> {
-        let (wasi_fs, root_inode) = Self::new_init(fs_backing, inodes)?;
-
-        for preopen_name in vfs_preopens {
-            let kind = Kind::Dir {
-                parent: root_inode.downgrade(),
-                path: PathBuf::from(preopen_name),
-                entries: Default::default(),
-            };
-            let rights = Rights::FD_ADVISE
-                | Rights::FD_TELL
-                | Rights::FD_SEEK
-                | Rights::FD_READ
-                | Rights::PATH_OPEN
-                | Rights::FD_READDIR
-                | Rights::PATH_READLINK
-                | Rights::PATH_FILESTAT_GET
-                | Rights::FD_FILESTAT_GET
-                | Rights::PATH_LINK_SOURCE
-                | Rights::PATH_RENAME_SOURCE
-                | Rights::POLL_FD_READWRITE
-                | Rights::SOCK_SHUTDOWN;
-            let inode = wasi_fs
-                .create_inode(inodes, kind, true, preopen_name.clone())
-                .map_err(|e| {
-                    format!(
-                        "Failed to create inode for preopened dir (name `{}`): WASI error code: {}",
-                        preopen_name, e
-                    )
-                })?;
-            let fd_flags = Fd::READ;
-            let fd = wasi_fs
-                .create_fd(rights, rights, Fdflags::empty(), fd_flags, inode.clone())
-                .map_err(|e| format!("Could not open fd for file {:?}: {}", preopen_name, e))?;
-            {
-                let mut guard = root_inode.write();
-                if let Kind::Root { entries } = guard.deref_mut() {
-                    let existing_entry = entries.insert(preopen_name.clone(), inode);
-                    if existing_entry.is_some() {
-                        return Err(format!(
-                            "Found duplicate entry for alias `{}`",
-                            preopen_name
-                        ));
-                    }
-                    assert!(existing_entry.is_none())
-                }
-            }
-            wasi_fs.preopen_fds.write().unwrap().push(fd);
-        }
-
-        for PreopenedDir {
-            path,
-            alias,
-            read,
-            write,
-            create,
-        } in preopens
-        {
-            debug!(
-                "Attempting to preopen {} with alias {:?}",
-                &path.to_string_lossy(),
-                &alias
-            );
-            let cur_dir_metadata = wasi_fs
-                .root_fs
-                .metadata(path)
-                .map_err(|e| format!("Could not get metadata for file {:?}: {}", path, e))?;
-
-            let kind = if cur_dir_metadata.is_dir() {
-                Kind::Dir {
-                    parent: root_inode.downgrade(),
-                    path: path.clone(),
-                    entries: Default::default(),
-                }
-            } else {
-                return Err(format!(
-                    "WASI only supports pre-opened directories right now; found \"{}\"",
-                    &path.to_string_lossy()
-                ));
-            };
-
-            let rights = {
-                // TODO: review tell' and fd_readwrite
-                let mut rights = Rights::FD_ADVISE | Rights::FD_TELL | Rights::FD_SEEK;
-                if *read {
-                    rights |= Rights::FD_READ
-                        | Rights::PATH_OPEN
-                        | Rights::FD_READDIR
-                        | Rights::PATH_READLINK
-                        | Rights::PATH_FILESTAT_GET
-                        | Rights::FD_FILESTAT_GET
-                        | Rights::PATH_LINK_SOURCE
-                        | Rights::PATH_RENAME_SOURCE
-                        | Rights::POLL_FD_READWRITE
-                        | Rights::SOCK_SHUTDOWN;
-                }
-                if *write {
-                    rights |= Rights::FD_DATASYNC
-                        | Rights::FD_FDSTAT_SET_FLAGS
-                        | Rights::FD_WRITE
-                        | Rights::FD_SYNC
-                        | Rights::FD_ALLOCATE
-                        | Rights::PATH_OPEN
-                        | Rights::PATH_RENAME_TARGET
-                        | Rights::PATH_FILESTAT_SET_SIZE
-                        | Rights::PATH_FILESTAT_SET_TIMES
-                        | Rights::FD_FILESTAT_SET_SIZE
-                        | Rights::FD_FILESTAT_SET_TIMES
-                        | Rights::PATH_REMOVE_DIRECTORY
-                        | Rights::PATH_UNLINK_FILE
-                        | Rights::POLL_FD_READWRITE
-                        | Rights::SOCK_SHUTDOWN;
-                }
-                if *create {
-                    rights |= Rights::PATH_CREATE_DIRECTORY
-                        | Rights::PATH_CREATE_FILE
-                        | Rights::PATH_LINK_TARGET
-                        | Rights::PATH_OPEN
-                        | Rights::PATH_RENAME_TARGET
-                        | Rights::PATH_SYMLINK;
-                }
-
-                rights
-            };
-            let inode = if let Some(alias) = &alias {
-                wasi_fs.create_inode(inodes, kind, true, alias.clone())
-            } else {
-                wasi_fs.create_inode(inodes, kind, true, path.to_string_lossy().into_owned())
-            }
-            .map_err(|e| {
-                format!(
-                    "Failed to create inode for preopened dir: WASI error code: {}",
-                    e
-                )
-            })?;
-            let fd_flags = {
-                let mut fd_flags = 0;
-                if *read {
-                    fd_flags |= Fd::READ;
-                }
-                if *write {
-                    // TODO: introduce API for finer grained control
-                    fd_flags |= Fd::WRITE | Fd::APPEND | Fd::TRUNCATE;
-                }
-                if *create {
-                    fd_flags |= Fd::CREATE;
-                }
-                fd_flags
-            };
-            let fd = wasi_fs
-                .create_fd(rights, rights, Fdflags::empty(), fd_flags, inode.clone())
-                .map_err(|e| format!("Could not open fd for file {:?}: {}", path, e))?;
-            {
-                let mut guard = root_inode.write();
-                if let Kind::Root { entries } = guard.deref_mut() {
-                    let key = if let Some(alias) = &alias {
-                        alias.clone()
-                    } else {
-                        path.to_string_lossy().into_owned()
-                    };
-                    let existing_entry = entries.insert(key.clone(), inode);
-                    if existing_entry.is_some() {
-                        return Err(format!("Found duplicate entry for alias `{}`", key));
-                    }
-                    assert!(existing_entry.is_none())
-                }
-            }
-            wasi_fs.preopen_fds.write().unwrap().push(fd);
-        }
-
+        let mut wasi_fs = Self::new_init(fs_backing, inodes)?;
+        wasi_fs.init_preopens = preopens.to_vec();
+        wasi_fs.init_vfs_preopens = vfs_preopens.to_vec();
+        wasi_fs.create_preopens(inodes, false)?;
         Ok(wasi_fs)
     }
 
@@ -701,7 +537,7 @@ impl WasiFs {
 
     /// Private helper function to init the filesystem, called in `new` and
     /// `new_with_preopen`
-    fn new_init(fs_backing: WasiFsRoot, inodes: &WasiInodes) -> Result<(Self, InodeGuard), String> {
+    fn new_init(fs_backing: WasiFsRoot, inodes: &WasiInodes) -> Result<Self, String> {
         debug!("Initializing WASI filesystem");
 
         let stat = Filestat {
@@ -725,46 +561,17 @@ impl WasiFs {
             current_dir: Mutex::new("/".to_string()),
             is_wasix: AtomicBool::new(false),
             root_fs: fs_backing,
-            root_inode: root_inode.clone(),
+            root_inode,
             has_unioned: Arc::new(Mutex::new(HashSet::new())),
+            init_preopens: Default::default(),
+            init_vfs_preopens: Default::default(),
         };
         wasi_fs.create_stdin(inodes);
         wasi_fs.create_stdout(inodes);
         wasi_fs.create_stderr(inodes);
+        wasi_fs.create_rootfd()?;
 
-        // create virtual root
-        let all_rights = ALL_RIGHTS;
-        // TODO: make this a list of positive rigths instead of negative ones
-        // root gets all right for now
-        let root_rights = all_rights
-            /*
-            & (!Rights::FD_WRITE)
-            & (!Rights::FD_ALLOCATE)
-            & (!Rights::PATH_CREATE_DIRECTORY)
-            & (!Rights::PATH_CREATE_FILE)
-            & (!Rights::PATH_LINK_SOURCE)
-            & (!Rights::PATH_RENAME_SOURCE)
-            & (!Rights::PATH_RENAME_TARGET)
-            & (!Rights::PATH_FILESTAT_SET_SIZE)
-            & (!Rights::PATH_FILESTAT_SET_TIMES)
-            & (!Rights::FD_FILESTAT_SET_SIZE)
-            & (!Rights::FD_FILESTAT_SET_TIMES)
-            & (!Rights::PATH_SYMLINK)
-            & (!Rights::PATH_UNLINK_FILE)
-            & (!Rights::PATH_REMOVE_DIRECTORY)
-            */;
-        let fd = wasi_fs
-            .create_fd(
-                root_rights,
-                root_rights,
-                Fdflags::empty(),
-                Fd::READ,
-                root_inode.clone(),
-            )
-            .map_err(|e| format!("Could not create root fd: {}", e))?;
-        wasi_fs.preopen_fds.write().unwrap().push(fd);
-
-        Ok((wasi_fs, root_inode))
+        Ok(wasi_fs)
     }
 
     /// This function is like create dir all, but it also opens it.
@@ -1123,7 +930,8 @@ impl WasiFs {
                                 let (pre_open_dir_fd, relative_path) = if link_value.is_relative() {
                                     self.path_into_pre_open_and_relative_path(&file)?
                                 } else {
-                                    unimplemented!("Absolute symlinks are not yet supported");
+                                    tracing::error!("Absolute symlinks are not yet supported");
+                                    return Err(Errno::Notsup);
                                 };
                                 loop_for_symlink = true;
                                 symlink_count += 1;
@@ -1663,6 +1471,22 @@ impl WasiFs {
         Ok(idx)
     }
 
+    pub fn make_max_fd(&self, fd: u32) {
+        loop {
+            let existing = self.next_fd.load(Ordering::SeqCst);
+            if existing >= fd {
+                return;
+            }
+            if self
+                .next_fd
+                .compare_exchange(existing, fd, Ordering::SeqCst, Ordering::Relaxed)
+                .is_ok()
+            {
+                break;
+            }
+        }
+    }
+
     pub fn create_fd_ext(
         &self,
         rights: Rights,
@@ -1722,7 +1546,7 @@ impl WasiFs {
         guard.lookup.remove(&ino).and_then(|a| Weak::upgrade(&a))
     }
 
-    fn create_stdout(&self, inodes: &WasiInodes) {
+    pub(crate) fn create_stdout(&self, inodes: &WasiInodes) {
         self.create_std_dev_inner(
             inodes,
             Box::<Stdout>::default(),
@@ -1733,7 +1557,7 @@ impl WasiFs {
         );
     }
 
-    fn create_stdin(&self, inodes: &WasiInodes) {
+    pub(crate) fn create_stdin(&self, inodes: &WasiInodes) {
         self.create_std_dev_inner(
             inodes,
             Box::<Stdin>::default(),
@@ -1744,7 +1568,7 @@ impl WasiFs {
         );
     }
 
-    fn create_stderr(&self, inodes: &WasiInodes) {
+    pub(crate) fn create_stderr(&self, inodes: &WasiInodes) {
         self.create_std_dev_inner(
             inodes,
             Box::<Stderr>::default(),
@@ -1755,7 +1579,215 @@ impl WasiFs {
         );
     }
 
-    fn create_std_dev_inner(
+    pub(crate) fn create_rootfd(&self) -> Result<(), String> {
+        // create virtual root
+        let all_rights = ALL_RIGHTS;
+        // TODO: make this a list of positive rigths instead of negative ones
+        // root gets all right for now
+        let root_rights = all_rights
+            /*
+            & (!Rights::FD_WRITE)
+            & (!Rights::FD_ALLOCATE)
+            & (!Rights::PATH_CREATE_DIRECTORY)
+            & (!Rights::PATH_CREATE_FILE)
+            & (!Rights::PATH_LINK_SOURCE)
+            & (!Rights::PATH_RENAME_SOURCE)
+            & (!Rights::PATH_RENAME_TARGET)
+            & (!Rights::PATH_FILESTAT_SET_SIZE)
+            & (!Rights::PATH_FILESTAT_SET_TIMES)
+            & (!Rights::FD_FILESTAT_SET_SIZE)
+            & (!Rights::FD_FILESTAT_SET_TIMES)
+            & (!Rights::PATH_SYMLINK)
+            & (!Rights::PATH_UNLINK_FILE)
+            & (!Rights::PATH_REMOVE_DIRECTORY)
+            */;
+        let fd = self
+            .create_fd(
+                root_rights,
+                root_rights,
+                Fdflags::empty(),
+                Fd::READ,
+                self.root_inode.clone(),
+            )
+            .map_err(|e| format!("Could not create root fd: {}", e))?;
+        self.preopen_fds.write().unwrap().push(fd);
+        Ok(())
+    }
+
+    pub(crate) fn create_preopens(
+        &self,
+        inodes: &WasiInodes,
+        ignore_duplicates: bool,
+    ) -> Result<(), String> {
+        for preopen_name in self.init_vfs_preopens.iter() {
+            let kind = Kind::Dir {
+                parent: self.root_inode.downgrade(),
+                path: PathBuf::from(preopen_name),
+                entries: Default::default(),
+            };
+            let rights = Rights::FD_ADVISE
+                | Rights::FD_TELL
+                | Rights::FD_SEEK
+                | Rights::FD_READ
+                | Rights::PATH_OPEN
+                | Rights::FD_READDIR
+                | Rights::PATH_READLINK
+                | Rights::PATH_FILESTAT_GET
+                | Rights::FD_FILESTAT_GET
+                | Rights::PATH_LINK_SOURCE
+                | Rights::PATH_RENAME_SOURCE
+                | Rights::POLL_FD_READWRITE
+                | Rights::SOCK_SHUTDOWN;
+            let inode = self
+                .create_inode(inodes, kind, true, preopen_name.clone())
+                .map_err(|e| {
+                    format!(
+                        "Failed to create inode for preopened dir (name `{}`): WASI error code: {}",
+                        preopen_name, e
+                    )
+                })?;
+            let fd_flags = Fd::READ;
+            let fd = self
+                .create_fd(rights, rights, Fdflags::empty(), fd_flags, inode.clone())
+                .map_err(|e| format!("Could not open fd for file {:?}: {}", preopen_name, e))?;
+            {
+                let mut guard = self.root_inode.write();
+                if let Kind::Root { entries } = guard.deref_mut() {
+                    let existing_entry = entries.insert(preopen_name.clone(), inode);
+                    if existing_entry.is_some() && !ignore_duplicates {
+                        return Err(format!(
+                            "Found duplicate entry for alias `{}`",
+                            preopen_name
+                        ));
+                    }
+                }
+            }
+            self.preopen_fds.write().unwrap().push(fd);
+        }
+
+        for PreopenedDir {
+            path,
+            alias,
+            read,
+            write,
+            create,
+        } in self.init_preopens.iter()
+        {
+            debug!(
+                "Attempting to preopen {} with alias {:?}",
+                &path.to_string_lossy(),
+                &alias
+            );
+            let cur_dir_metadata = self
+                .root_fs
+                .metadata(path)
+                .map_err(|e| format!("Could not get metadata for file {:?}: {}", path, e))?;
+
+            let kind = if cur_dir_metadata.is_dir() {
+                Kind::Dir {
+                    parent: self.root_inode.downgrade(),
+                    path: path.clone(),
+                    entries: Default::default(),
+                }
+            } else {
+                return Err(format!(
+                    "WASI only supports pre-opened directories right now; found \"{}\"",
+                    &path.to_string_lossy()
+                ));
+            };
+
+            let rights = {
+                // TODO: review tell' and fd_readwrite
+                let mut rights = Rights::FD_ADVISE | Rights::FD_TELL | Rights::FD_SEEK;
+                if *read {
+                    rights |= Rights::FD_READ
+                        | Rights::PATH_OPEN
+                        | Rights::FD_READDIR
+                        | Rights::PATH_READLINK
+                        | Rights::PATH_FILESTAT_GET
+                        | Rights::FD_FILESTAT_GET
+                        | Rights::PATH_LINK_SOURCE
+                        | Rights::PATH_RENAME_SOURCE
+                        | Rights::POLL_FD_READWRITE
+                        | Rights::SOCK_SHUTDOWN;
+                }
+                if *write {
+                    rights |= Rights::FD_DATASYNC
+                        | Rights::FD_FDSTAT_SET_FLAGS
+                        | Rights::FD_WRITE
+                        | Rights::FD_SYNC
+                        | Rights::FD_ALLOCATE
+                        | Rights::PATH_OPEN
+                        | Rights::PATH_RENAME_TARGET
+                        | Rights::PATH_FILESTAT_SET_SIZE
+                        | Rights::PATH_FILESTAT_SET_TIMES
+                        | Rights::FD_FILESTAT_SET_SIZE
+                        | Rights::FD_FILESTAT_SET_TIMES
+                        | Rights::PATH_REMOVE_DIRECTORY
+                        | Rights::PATH_UNLINK_FILE
+                        | Rights::POLL_FD_READWRITE
+                        | Rights::SOCK_SHUTDOWN;
+                }
+                if *create {
+                    rights |= Rights::PATH_CREATE_DIRECTORY
+                        | Rights::PATH_CREATE_FILE
+                        | Rights::PATH_LINK_TARGET
+                        | Rights::PATH_OPEN
+                        | Rights::PATH_RENAME_TARGET
+                        | Rights::PATH_SYMLINK;
+                }
+
+                rights
+            };
+            let inode = if let Some(alias) = &alias {
+                self.create_inode(inodes, kind, true, alias.clone())
+            } else {
+                self.create_inode(inodes, kind, true, path.to_string_lossy().into_owned())
+            }
+            .map_err(|e| {
+                format!(
+                    "Failed to create inode for preopened dir: WASI error code: {}",
+                    e
+                )
+            })?;
+            let fd_flags = {
+                let mut fd_flags = 0;
+                if *read {
+                    fd_flags |= Fd::READ;
+                }
+                if *write {
+                    // TODO: introduce API for finer grained control
+                    fd_flags |= Fd::WRITE | Fd::APPEND | Fd::TRUNCATE;
+                }
+                if *create {
+                    fd_flags |= Fd::CREATE;
+                }
+                fd_flags
+            };
+            let fd = self
+                .create_fd(rights, rights, Fdflags::empty(), fd_flags, inode.clone())
+                .map_err(|e| format!("Could not open fd for file {:?}: {}", path, e))?;
+            {
+                let mut guard = self.root_inode.write();
+                if let Kind::Root { entries } = guard.deref_mut() {
+                    let key = if let Some(alias) = &alias {
+                        alias.clone()
+                    } else {
+                        path.to_string_lossy().into_owned()
+                    };
+                    let existing_entry = entries.insert(key.clone(), inode);
+                    if existing_entry.is_some() && !ignore_duplicates {
+                        return Err(format!("Found duplicate entry for alias `{}`", key));
+                    }
+                }
+            }
+            self.preopen_fds.write().unwrap().push(fd);
+        }
+
+        Ok(())
+    }
+
+    pub(crate) fn create_std_dev_inner(
         &self,
         inodes: &WasiInodes,
         handle: Box<dyn VirtualFile + Send + Sync + 'static>,
