@@ -47,6 +47,7 @@ pub mod net;
 pub mod capabilities;
 pub mod fs;
 pub mod http;
+pub mod journal;
 mod rewind;
 pub mod runners;
 pub mod runtime;
@@ -57,6 +58,8 @@ mod utils;
 /// WAI based bindings.
 mod bindings;
 
+use std::sync::Arc;
+
 #[allow(unused_imports)]
 use bytes::{Bytes, BytesMut};
 use os::task::control_plane::ControlPlaneError;
@@ -103,7 +106,7 @@ pub use crate::{
     utils::is_wasix_module,
     utils::{
         get_wasi_version, get_wasi_versions, is_wasi_module,
-        store::{capture_snapshot, restore_snapshot, InstanceSnapshot},
+        store::{capture_instance_snapshot, restore_instance_snapshot, InstanceSnapshot},
         WasiVersion,
     },
 };
@@ -120,6 +123,8 @@ pub enum WasiError {
     UnknownWasiVersion,
 }
 
+pub type WasiResult<T> = Result<Result<T, Errno>, WasiError>;
+
 #[deny(unused, dead_code)]
 #[derive(Error, Debug)]
 pub enum SpawnError {
@@ -203,6 +208,8 @@ pub enum WasiRuntimeError {
     Runtime(#[from] RuntimeError),
     #[error("Memory access error")]
     Thread(#[from] WasiThreadError),
+    #[error("{0}")]
+    Anyhow(#[from] Arc<anyhow::Error>),
 }
 
 impl WasiRuntimeError {
@@ -717,8 +724,6 @@ fn import_object_for_all_wasi_versions(
             } else {
                 Box::new(stub_initializer) as ModuleInitializer
             };
-
-            let init = init;
         } else {
             // Prevents unused warning.
             let _ = module;
@@ -779,3 +784,18 @@ fn mem_error_to_wasi(err: MemoryAccessError) -> Errno {
         _ => Errno::Unknown,
     }
 }
+
+/// Run a synchronous function that would normally be blocking.
+///
+/// When the `sys-thread` feature is enabled, this will call
+/// [`tokio::task::block_in_place()`]. Otherwise, it calls the function
+/// immediately.
+pub(crate) fn block_in_place<Ret>(thunk: impl FnOnce() -> Ret) -> Ret {
+    cfg_if::cfg_if! {
+        if #[cfg(feature = "sys-thread")] {
+            tokio::task::block_in_place(thunk)
+        } else {
+            thunk()
+        }
+    }
+}
