@@ -16,32 +16,62 @@ use crate::syscalls::*;
 ///     The number of bytes to read from `new_path`
 #[instrument(level = "debug", skip_all, fields(%fd, old_path = field::Empty, new_path = field::Empty), ret)]
 pub fn path_symlink<M: MemorySize>(
-    ctx: FunctionEnvMut<'_, WasiEnv>,
+    mut ctx: FunctionEnvMut<'_, WasiEnv>,
     old_path: WasmPtr<u8, M>,
     old_path_len: M::Offset,
     fd: WasiFd,
     new_path: WasmPtr<u8, M>,
     new_path_len: M::Offset,
-) -> Errno {
+) -> Result<Errno, WasiError> {
     let env = ctx.data();
     let (memory, mut state, inodes) = unsafe { env.get_memory_and_wasi_state_and_inodes(&ctx, 0) };
-    let mut old_path_str = unsafe { get_input_str!(&memory, old_path, old_path_len) };
+    let mut old_path_str = unsafe { get_input_str_ok!(&memory, old_path, old_path_len) };
     Span::current().record("old_path", old_path_str.as_str());
-    let mut new_path_str = unsafe { get_input_str!(&memory, new_path, new_path_len) };
+    let mut new_path_str = unsafe { get_input_str_ok!(&memory, new_path, new_path_len) };
     Span::current().record("new_path", new_path_str.as_str());
     old_path_str = ctx.data().state.fs.relative_path_to_absolute(old_path_str);
     new_path_str = ctx.data().state.fs.relative_path_to_absolute(new_path_str);
-    let base_fd = wasi_try!(state.fs.get_fd(fd));
+
+    wasi_try_ok!(path_symlink_internal(
+        &mut ctx,
+        &old_path_str,
+        fd,
+        &new_path_str
+    ));
+    let env = ctx.data();
+
+    #[cfg(feature = "journal")]
+    if env.enable_journal {
+        JournalEffector::save_path_symlink(&mut ctx, old_path_str, fd, new_path_str).map_err(
+            |err| {
+                tracing::error!("failed to save path symbolic link event - {}", err);
+                WasiError::Exit(ExitCode::Errno(Errno::Fault))
+            },
+        )?;
+    }
+
+    Ok(Errno::Success)
+}
+
+pub fn path_symlink_internal(
+    ctx: &mut FunctionEnvMut<'_, WasiEnv>,
+    old_path: &str,
+    fd: WasiFd,
+    new_path: &str,
+) -> Result<(), Errno> {
+    let env = ctx.data();
+    let (memory, mut state, inodes) = unsafe { env.get_memory_and_wasi_state_and_inodes(&ctx, 0) };
+
+    let base_fd = state.fs.get_fd(fd)?;
     if !base_fd.rights.contains(Rights::PATH_SYMLINK) {
-        return Errno::Access;
+        return Err(Errno::Access);
     }
 
     // get the depth of the parent + 1 (UNDER INVESTIGATION HMMMMMMMM THINK FISH ^ THINK FISH)
-    let old_path_path = std::path::Path::new(&old_path_str);
-    let (source_inode, _) =
-        wasi_try!(state
-            .fs
-            .get_parent_inode_at_path(inodes, fd, old_path_path, true));
+    let old_path_path = std::path::Path::new(old_path);
+    let (source_inode, _) = state
+        .fs
+        .get_parent_inode_at_path(inodes, fd, old_path_path, true)?;
     let depth = state.fs.path_depth_from_fd(fd, source_inode);
 
     // depth == -1 means folder is not relative. See issue #3233.
@@ -50,11 +80,11 @@ pub fn path_symlink<M: MemorySize>(
         Err(_) => -1,
     };
 
-    let new_path_path = std::path::Path::new(&new_path_str);
+    let new_path_path = std::path::Path::new(new_path);
     let (target_parent_inode, entry_name) =
-        wasi_try!(state
+        state
             .fs
-            .get_parent_inode_at_path(inodes, fd, new_path_path, true));
+            .get_parent_inode_at_path(inodes, fd, new_path_path, true)?;
 
     // short circuit if anything is wrong, before we create an inode
     {
@@ -62,21 +92,21 @@ pub fn path_symlink<M: MemorySize>(
         match guard.deref() {
             Kind::Dir { entries, .. } => {
                 if entries.contains_key(&entry_name) {
-                    return Errno::Exist;
+                    return Err(Errno::Exist);
                 }
             }
-            Kind::Root { .. } => return Errno::Notcapable,
+            Kind::Root { .. } => return Err(Errno::Notcapable),
             Kind::Socket { .. }
             | Kind::Pipe { .. }
             | Kind::EventNotifications { .. }
-            | Kind::Epoll { .. } => return Errno::Inval,
+            | Kind::Epoll { .. } => return Err(Errno::Inval),
             Kind::File { .. } | Kind::Symlink { .. } | Kind::Buffer { .. } => {
                 unreachable!("get_parent_inode_at_path returned something other than a Dir or Root")
             }
         }
     }
 
-    let mut source_path = std::path::Path::new(&old_path_str);
+    let mut source_path = std::path::Path::new(old_path);
     let mut relative_path = std::path::PathBuf::new();
     for _ in 0..depth {
         relative_path.push("..");
@@ -85,7 +115,7 @@ pub fn path_symlink<M: MemorySize>(
 
     let kind = Kind::Symlink {
         base_po_dir: fd,
-        path_to_symlink: std::path::PathBuf::from(new_path_str),
+        path_to_symlink: std::path::PathBuf::from(new_path),
         relative_path,
     };
     let new_inode =
@@ -103,5 +133,5 @@ pub fn path_symlink<M: MemorySize>(
         }
     }
 
-    Errno::Success
+    Ok(())
 }
